﻿-- Author      : Nogarder
-- Create Date : 2013/6/8 13:31:46
local table_getn=table.getn
local table_insert=table.insert
local next=next
local UnitGUID=UnitGUID
local UnitExists=UnitExists
local string_find=string.find
local string_sub=string.sub
local tonumber=tonumber
local NPA_GetTime=NPA.GetTime
--
local NPA_NamePlateDataParser_AddElements=NPA.NamePlateDataParser.AddElements
local NPA_TextureManager_ManageTexture2=NPA.TextureManager.ManageTexture2
local NPA_NamePlateModuleManager_ZipModule=NPA.NamePlateModuleManager.ZipModule
local NPA_NamePlateModuleManager_UnzipModule=NPA.NamePlateModuleManager.UnzipModule
--
local UpdateScale=function(modulemain,prioritylist,count)
	for i=1,count do
		if(prioritylist[i])then
			modulemain:SetScale(prioritylist[i])
			return
		end
	end
end
--
NPA.NamePlateModuleManager.RegisterDataModule("MainFrame","Scale",function(framedata,modulemain,settings)
	local prioritylist={}
	--
	local TargetCondition={}
	local NameCondition={}
	local PartNameCondition={}
	local UnitIDCondition={}
	local SummonedCondition={}
	local PlayerCondition={}
	local NonPlayerCondition={}
	local WithAuraCondition={}
	local WithoutAuraCondition={}
	local HealthMoreCondition={}
	local HealthLessCondition={}
	--
	local count=table_getn(settings)
	--add event elements
 	for i=1,count do
		local setting=settings[i]
		if(setting.ConditionType=="Default")then
			--using default alpha
			NPA_NamePlateDataParser_AddElements(framedata,"Scale","Scale_Updated",function(framedata,unitdata)
				prioritylist[i]=1
				UpdateScale(modulemain,prioritylist,count)
			end)
		elseif(setting.ConditionType=="Any")then
			--any would always be using
			prioritylist[i]=setting.Scale
		elseif(setting.ConditionType=="Quest")then
			NPA_NamePlateDataParser_AddElements(framedata,"Scale","Quest_Updated",function(framedata,_,data)
				if(data)then
					prioritylist[i]=setting.Scale
				else
					prioritylist[i]=nil
				end
				UpdateScale(modulemain,prioritylist,count)
			end)
		elseif(setting.ConditionType=="Name")then
			table_insert(NameCondition,i)
		elseif(setting.ConditionType=="PartName")then
			table_insert(PartNameCondition,i)
		elseif(setting.ConditionType=="UnitID")then
			table_insert(UnitIDCondition,i)
		elseif(setting.ConditionType=="Summoned")then
			table_insert(SummonedCondition,i)
		elseif(setting.ConditionType=="Player")then
			table_insert(PlayerCondition,i)
		elseif(setting.ConditionType=="NonPlayer")then
			table_insert(NonPlayerCondition,i)
		elseif(setting.ConditionType=="Target")then
			table_insert(TargetCondition,i)
		elseif(setting.ConditionType=="WithAura")then
			table_insert(WithAuraCondition,i)
		elseif(setting.ConditionType=="WithoutAura")then
			table_insert(WithoutAuraCondition,i)
		elseif(setting.ConditionType=="HealthMore")then
			table_insert(HealthMoreCondition,i)
		elseif(setting.ConditionType=="HealthLess")then
			table_insert(HealthLessCondition,i)
		end
	end
	--default scale
	count=count+1
	prioritylist[count]=1
	--target
	if(next(TargetCondition))then
		NPA_NamePlateDataParser_AddElements(framedata,"Scale","ModuleMain_OnUpdate",function(framedata)
			local guid=framedata.GUID
			if(not guid)then
				return
			end
			--
			local update
			for i=1,table_getn(TargetCondition)do
				local index=TargetCondition[i]
				--check priority
				local setting=settings[index]
				local check=(UnitExists(setting.Arg) and guid==UnitGUID(setting.Arg))
				if(prioritylist[index]==nil and check)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(prioritylist[index]~=nil and not check)then
					prioritylist[index]=nil
					update=true
				end
			end
			--
			if(update)then
				UpdateScale(modulemain,prioritylist,count)
			end
		end)
	end
	--name
	if(next(NameCondition) or next(PartNameCondition))then
		NPA_NamePlateDataParser_AddElements(framedata,"Scale","Name_Updated",function(framedata,unitdata)
			local name=unitdata.Name
			if(not name)then
				--in case no name yet
				return
			end
			--
			local update
			--name check
			for i=1,table_getn(NameCondition)do
				local index=NameCondition[i]
				--check priority
				local setting=settings[index]
				if(prioritylist[index]==nil and name==setting.Arg)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(prioritylist[index]~=nil and name~=setting.Arg)then
					prioritylist[index]=nil
					update=true
				end
			end
			--part name check
			for i=1,table_getn(PartNameCondition)do
				local index=PartNameCondition[i]
				--check priority
				local setting=settings[index]
				local found=string_find(name,setting.Arg)
				if(prioritylist[index]==nil and found)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(prioritylist[index]~=nil and not found)then
					prioritylist[index]=nil
					update=true
				end
			end
			--
			if(update)then
				UpdateScale(modulemain,prioritylist,count)
			end
		end)
	end
	--unitid
	if(next(UnitIDCondition))then
		NPA_NamePlateDataParser_AddElements(framedata,"Scale","GUID_Attached",function(framedata)
			--this will get an unit id for npc
			local unitid=tonumber(string_sub(framedata.GUID,7,10),16)
			local update
			--name check
			for i=1,table_getn(UnitIDCondition)do
				local index=UnitIDCondition[i]
				--check priority
				local setting=settings[index]
				if(prioritylist[index]==nil and unitid==setting.Arg)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(prioritylist[index]~=nil and unitid~=setting.Arg)then
					prioritylist[index]=nil
					update=true
				end
			end
			--
			if(update)then
				UpdateScale(modulemain,prioritylist,count)
			end
		end)
	end
	--summoned
	if(next(SummonedCondition))then
		NPA_NamePlateDataParser_AddElements(framedata,"Scale","Summoned_Updated",function(framedata,unitdata)
			--filter out sometime the sourceunitdata is not avaliable
			if(not unitdata.SummonedByUnitData)then
				return
			end
			--
			local update
			--check summoned
			local sourcefaction=unitdata.SummonedByUnitData.Faction
			for i=1,table_getn(SummonedCondition)do
				local index=SummonedCondition[i]
				--check priority
				local setting=settings[index]
				--faction, 1=friendly,2=neutral,3=hostile
				if(prioritylist[index]==nil and tonumber(setting.Arg)==sourcefaction)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(prioritylist[index]~=nil and tonumber(setting.Arg)~=sourcefaction)then
					prioritylist[index]=nil
					update=true
				end
			end
			--
			if(update)then
				UpdateScale(modulemain,prioritylist,count)
			end
		end)
	end
	--player
	if(next(PlayerCondition))then
		NPA_NamePlateDataParser_AddElements(framedata,"Scale","Type_Updated",function(framedata,unitdata)
			--filter out sometime the sourceunitdata is not avaliable
			local update
			--check summoned
			local faction=framedata.UnitData.Faction
			local atype=framedata.UnitData.Type
			--data.Type= 1=player, 2=totem, 3=creature, 4=vendor, nil=unknow
			if(atype==1)then
				for i=1,table_getn(PlayerCondition)do
					local index=PlayerCondition[i]
					--check priority
					local setting=settings[index]
					--faction, 1=friendly,2=neutral,3=hostile
					if(prioritylist[index]==nil and tonumber(setting.Arg)==faction)then
						prioritylist[index]=setting.Scale
						update=true
					elseif(prioritylist[index]~=nil and tonumber(setting.Arg)~=faction)then
						prioritylist[index]=nil
						update=true
					end
				end
			elseif(atype==nil)then
				for i=1,table_getn(NonPlayerCondition)do
					local index=NonPlayerCondition[i]
					--check priority
					local setting=settings[index]
					--faction, 1=friendly,2=neutral,3=hostile
					if(prioritylist[index]==nil and tonumber(setting.Arg)==faction)then
						prioritylist[index]=setting.Scale
						update=true
					elseif(prioritylist[index]~=nil and tonumber(setting.Arg)~=faction)then
						prioritylist[index]=nil
						update=true
					end
				end
			end
			--
			if(update)then
				UpdateScale(modulemain,prioritylist,count)
			end
		end)
	end
	--aura
	if(next(WithAuraCondition) or next(WithoutAuraCondition))then
		NPA_NamePlateDataParser_AddElements(framedata,"Scale","Aura_Updated",function(framedata,unitdata)
			if(not unitdata.Aura)then
				return
			end
			--
			local nTime=NPA_GetTime()
			--
			local update
			local auralist=framedata.UnitData.Aura
			--check with list
			for i=1,table_getn(WithAuraCondition)do
				local index=WithAuraCondition[i]
				local setting=settings[index]
				local checked
				for i=1,table_getn(auralist)do
					--check with list
					if(auralist[i].SpellId==tonumber(setting.Arg) and auralist[i].ExpirationTime>nTime)then
						checked=true
						break
					end
				end
				if(checked==true and prioritylist[index]==nil)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(checked==nil and prioritylist[index]~=nil)then
					prioritylist[index]=nil
					update=true
				end
			end
			--check without list , just the revers version of with
			for i=1,table_getn(WithoutAuraCondition)do
				local index=WithoutAuraCondition[i]
				local setting=settings[index]
				local checked
				for i=1,table_getn(auralist)do
					--check with list
					if(auralist[i].SpellId==tonumber(setting.Arg) and auralist[i].ExpirationTime>nTime)then
						checked=true
						break
					end
				end
				if(checked==nil and prioritylist[index]==nil)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(checked==true and prioritylist[index]~=nil)then
					prioritylist[index]=nil
					update=true
				end
			end
			--
			if(update)then
				UpdateScale(modulemain,prioritylist,count)
			end
		end)
	end
	--health
	if(next(HealthMoreCondition) or next(HealthLessCondition))then
		NPA_NamePlateDataParser_AddElements(framedata,"Scale","Health_Updated",function(framedata,unitdata)
			if((not unitdata.Health) or (not unitdata.HealthMax))then
				return
			end
			local healthpercent=unitdata.Health/unitdata.HealthMax
			--check "more" condition
			for i=1,table_getn(HealthMoreCondition)do
				local index=HealthMoreCondition[i]
				local setting=settings[index]
				local checked
				--check with list
				if(healthpercent>=tonumber(setting.Arg)/100)then
					checked=true
				end
				if(checked==true and prioritylist[index]==nil)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(checked==nil and prioritylist[index]~=nil)then
					prioritylist[index]=nil
					update=true
				end
			end
			--check "less" condition
			for i=1,table_getn(HealthLessCondition)do
				local index=HealthLessCondition[i]
				local setting=settings[index]
				local checked
				--check with list
				if(healthpercent<=tonumber(setting.Arg)/100)then
					checked=true
				end
				if(checked==true and prioritylist[index]==nil)then
					prioritylist[index]=setting.Scale
					update=true
				elseif(checked==nil and prioritylist[index]~=nil)then
					prioritylist[index]=nil
					update=true
				end
			end
			--
			if(update)then
				UpdateScale(modulemain,prioritylist,count)
			end
		end)
	end
	--update once
	UpdateScale(modulemain,prioritylist,count)
end)
